const float timer_timeout = 0.5;
bool button_pressed = false;
bool blue_collision = false;
bool orange_collision = false;
bool reset_collision = false;
bool help_collision = false;
bool done_collision = false;
bool portals_enabled = false;
Vec3 initial_box_pos = Vec3(0, 0, 0);

class TeleportTimer
{
    void onTimerComplete()
    {
        if (button_pressed && !blue_collision && !orange_collision && 
            !reset_collision && !help_collision && !done_collision)
        {
            button_pressed = false;
        }

        blue_collision = false;
        orange_collision = false;
        reset_collision = false;
        help_collision = false;
        done_collision = false;

        Utils::logInfo("onTimerComplete");
        Utils::TimeoutCallback@ timerDelegate = Utils::TimeoutCallback(onTimerComplete);
        Utils::setTimeoutDelegate(timerDelegate, timer_timeout);
    }
}

void printHelp(bool overlay)
{
    const string message = "Put green box near the green plane. \nThen press DONE button.";
    
    if (overlay)
    {
        GUI::clearOverlayMessages();
        GUI::displayOverlayMessage(message);
    }
    else
    {
        GUI::displayModalMessage(message);
    }
}

void enablePortals()
{
    if (portals_enabled)
        return;

    portals_enabled = true;

    Track::TrackObject@ blue_portal = Track::getTrackObject("", "blue_portal");
    Vec3 blue_text_pos = blue_portal.getCenterPosition();
    blue_text_pos = Vec3(blue_text_pos.getX(), blue_text_pos.getY() + 2.0, blue_text_pos.getZ());
    Track::createTextBillboard("Blue Portal", blue_text_pos);
    blue_portal.setEnabled(true);

    Track::TrackObject@ orange_portal = Track::getTrackObject("", "orange_portal");
    Vec3 orange_text_pos = orange_portal.getCenterPosition();
    orange_text_pos = Vec3(orange_text_pos.getX(), orange_text_pos.getY() + 2.0, orange_text_pos.getZ());
    Track::createTextBillboard("Orange Portal", orange_text_pos);
    orange_portal.setEnabled(true);
}

void onStart()
{   
    Track::TrackObject@ green_box = Track::getTrackObject("", "green_box");
    initial_box_pos = green_box.getOrigin();

    Track::TrackObject@ blue_portal = Track::getTrackObject("", "blue_portal");
    blue_portal.setEnabled(false);

    Track::TrackObject@ orange_portal = Track::getTrackObject("", "orange_portal");
    orange_portal.setEnabled(false);

    Track::TrackObject@ reset = Track::getTrackObject("", "reset");
    Vec3 reset_text_pos = reset.getCenterPosition();
    reset_text_pos = Vec3(reset_text_pos.getX(), reset_text_pos.getY() + 2.0, reset_text_pos.getZ());
    Track::createTextBillboard("Reset", reset_text_pos);

    Track::TrackObject@ help = Track::getTrackObject("", "help");
    Vec3 help_text_pos = help.getCenterPosition();
    help_text_pos = Vec3(help_text_pos.getX(), help_text_pos.getY() + 2.0, help_text_pos.getZ());
    Track::createTextBillboard("Help", help_text_pos);

    Track::TrackObject@ done = Track::getTrackObject("", "done");
    Vec3 done_text_pos = done.getCenterPosition();
    done_text_pos = Vec3(done_text_pos.getX(), done_text_pos.getY() + 2.0, done_text_pos.getZ());
    Track::createTextBillboard("Done", done_text_pos);

    TeleportTimer@ timer = TeleportTimer();
    Utils::TimeoutCallback@ timerDelegate = Utils::TimeoutCallback(timer.onTimerComplete);
    Utils::setTimeoutDelegate(timerDelegate, timer_timeout);

    printHelp(false);
}

void bluePortalCollision(int idKart, const string libraryInstance, const string objID)
{
    blue_collision = true;

    if (button_pressed)
        return;

    button_pressed = true;

    Utils::logInfo("bluePortalCollision " + libraryInstance + " " + objID);
    Track::TrackObject@ obj = Track::getTrackObject("", "orange_portal");
    Vec3 pos = obj.getCenterPosition();
    Utils::logInfo("orange portal pos " + pos.getX() + " " +  pos.getY() + " " + pos.getZ());
    Kart::teleportExact(idKart, Vec3(pos.getX(), pos.getY(), pos.getZ()));
}

void orangePortalCollision(int idKart, const string libraryInstance, const string objID)
{
    orange_collision = true;

    if (button_pressed)
        return;

    button_pressed = true;

    Utils::logInfo("orangePortalCollision");
    Track::TrackObject@ obj = Track::getTrackObject("", "blue_portal");
    Vec3 pos = obj.getCenterPosition();
    Utils::logInfo("blue portal pos " + pos.getX() + " " +  pos.getY() + " " + pos.getZ());
    Kart::teleportExact(idKart, Vec3(pos.getX(), pos.getY(), pos.getZ()));
}

void resetCollision(int idKart, const string libraryInstance, const string objID)
{
    reset_collision = true;

    if (button_pressed)
        return;

    button_pressed = true;

    Utils::logInfo("resetCollision");
    Track::TrackObject@ green_box = Track::getTrackObject("", "green_box");
    Vec3 green_box_pos = green_box.getCenterPosition();
    Utils::logInfo("green box pos " + green_box_pos.getX() + " " +  
                                      green_box_pos.getY() + " " + 
                                      green_box_pos.getZ());
    green_box.setEnabled(true);
    green_box.reset();

    GUI::clearOverlayMessages();
    GUI::displayOverlayMessage("OK");
}

void helpCollision(int idKart, const string libraryInstance, const string objID)
{
//    help_collision = true;
//
//    if (button_pressed)
//        return;
//
//    button_pressed = true;
//
//    Utils::logInfo("helpCollision");
//    //printHelp();
}

void helpTrigger(int idKart)
{
    Utils::logInfo("helpTrigger");
    printHelp(true);
}

void doneCollision(int idKart, const string libraryInstance, const string objID)
{
    done_collision = true;

    if (button_pressed)
        return;

    button_pressed = true;

    Utils::logInfo("doneCollision");

    Track::TrackObject@ green_box = Track::getTrackObject("", "green_box");
    Vec3 green_box_pos = green_box.getCenterPosition();
    Utils::logInfo("green box pos " + green_box_pos.getX() + " " +  
                                      green_box_pos.getY() + " " + 
                                      green_box_pos.getZ());

    Track::TrackObject@ green_plane = Track::getTrackObject("", "green_plane");
    Vec3 green_plane_pos = green_plane.getCenterPosition();
    Utils::logInfo("green plane pos " + green_plane_pos.getX() + " " +  
                                        green_plane_pos.getY() + " " + 
                                        green_plane_pos.getZ());

    if (green_box_pos.getX() - green_plane_pos.getX() < 2 && 
        green_box_pos.getX() - green_plane_pos.getX() > -2 &&
        green_box_pos.getZ() - green_plane_pos.getZ() < 2 && 
        green_box_pos.getZ() - green_plane_pos.getZ() > -2)
    {
        GUI::clearOverlayMessages();
        GUI::displayOverlayMessage("GOOD! :)\nEnjoy the portals!");
        enablePortals();
    }
    else
    {
        GUI::clearOverlayMessages();
        GUI::displayOverlayMessage("BAD :(");
        //Vec3 pos = Kart::getLocation(idKart);
        //Physics::createExplosion(pos);
        Kart::squash(idKart, 5);
        green_box.setEnabled(false);
    }
}

void greenBoxCollision(int idKart, const string libraryInstance, const string objID)
{

}

void greenPlaneCollision(int idKart, const string libraryInstance, const string objID)
{

}